@@ -11,7 +11,7 @@ from collections_extended import RangeMap
 from pydantic import BaseModel, Field
 from pymultirole_plugins.v1.formatter import FormatterParameters, FormatterBase
 from pymultirole_plugins.v1.processor import ProcessorBase, ProcessorParameters
-from pymultirole_plugins.v1.schema import Document, Sentence
+from pymultirole_plugins.v1.schema import Document, Sentence, AltText
 from spacy.cli.download import download_model, get_compatibility, get_version
 from spacy.errors import OLD_MODEL_SHORTCUTS
 from spacy.language import Language
@@ -23,59 +23,54 @@ from wasabi import msg
 # _home = os.path.expanduser('~')
 # xdg_cache_home = os.environ.get('XDG_CACHE_HOME') or os.path.join(_home, '.cache')
 
+
 class TextRankAlgo(str, Enum):
-    textrank = 'textrank'
-    positionrank = 'positionrank'
+    textrank = "textrank"
+    positionrank = "positionrank"
     # biasedtextrank = 'biasedtextrank'
 
 
 class TextRankSummarizerParameters(FormatterParameters):
-    as_metadata: str = Field(None, description="""If defined generate the summary as a metadata of the input document,
-    if not replace the text of the input document.""")
-    lang: str = Field("en",
-                      description="Name of the 2-letter language of the documents")
-    algo: TextRankAlgo = Field(TextRankAlgo.textrank, description="""The textgraph algorithms to use<br />
+    as_altText: str = Field(
+        None,
+        description="""If defined generate the summary as an alternative text of the input document,
+    if not replace the text of the input document.""",
+    )
+    lang: str = Field(
+        "en", description="Name of the 2-letter language of the documents"
+    )
+    algo: TextRankAlgo = Field(
+        TextRankAlgo.textrank,
+        description="""The textgraph algorithms to use<br />
     <li>`textrank` TextRank by [mihalcea04textrank](https://web.eecs.umich.edu/~mihalcea/papers/mihalcea.emnlp04.pdf)<br />
-    <li>`positionrank` PositionRank by [florescuc17](https://www.aclweb.org/anthology/P17-1102.pdf)<br />""")
+    <li>`positionrank` PositionRank by [florescuc17](https://www.aclweb.org/anthology/P17-1102.pdf)<br />""",
+    )
     # <li>`biasedtextrank` Biased TextRank by [kazemi-etal-2020-biased](https://www.aclweb.org/anthology/2020.coling-main.144.pdf)<br />""")
-    algo: TextRankAlgo = Field(TextRankAlgo.textrank,
-                               description="Name of the 2-letter language of the documents")
-    num_sentences: float = Field(0.25, description="""Number of sentences of the summary:<br/>
+    algo: TextRankAlgo = Field(
+        TextRankAlgo.textrank,
+        description="Name of the 2-letter language of the documents",
+    )
+    num_sentences: float = Field(
+        0.25,
+        description="""Number of sentences of the summary:<br/>
         <li>If float in the range [0.0, 1.0], then consider num_sentences as a percentage of the original number
-        of sentences of the document.""")
-    num_phrases: int = Field(10, description="Maximum number of top-ranked phrases to use in the distance vectors.")
-    preserve_order: bool = Field(False, description="""Flag to preserve the order of sentences as they originally
-    occurred in the source text; defaults to `False`""")
+        of sentences of the document.""",
+    )
+    num_phrases: int = Field(
+        10,
+        description="Maximum number of top-ranked phrases to use in the distance vectors.",
+    )
+    preserve_order: bool = Field(
+        False,
+        description="""Flag to preserve the order of sentences as they originally
+    occurred in the source text; defaults to `False`""",
+    )
 
 
 class TextRankSummarizerFormatter(FormatterBase, ProcessorBase, ABC):
-    """"A graphbased ranking model for text processing. Extractive sentence summarization.
-    """
+    """A graphbased ranking model for text processing. Extractive sentence summarization."""
 
     # cache_dir = os.path.join(xdg_cache_home, 'trankit')
-    def _check_sentences(self, nlp, algo: str, document: Document):
-        doc: Doc = None
-        if not document.sentences:
-            doc = nlp(document.text)
-        else:
-            textrank = nlp.get_pipe(algo)
-            parser = nlp.get_pipe("parser")
-            with nlp.select_pipes(disable=["parser", algo]):
-                doc = nlp(document.text)
-                sent_map = RangeMap()
-                for i, s in enumerate(document.sentences):
-                    if s.end > s.start:
-                        sent_map[s.start:s.end] = i
-                current = -1
-                for token in doc:
-                    if token.idx in sent_map:
-                        idsent = sent_map[token.idx]
-                        if idsent != current:
-                            token.is_sent_start = True
-                            current = idsent
-                if doc.has_annotation("SENT_START"):
-                    doc = textrank(parser(doc))
-        return doc
 
     def _summarize(self, document: Document, parameters: FormatterParameters) -> str:
         def int_float(v: float):
@@ -83,13 +78,18 @@ class TextRankSummarizerFormatter(FormatterBase, ProcessorBase, ABC):
                 return v
             return int(abs(v))
 
-        params: TextRankSummarizerParameters = \
-            cast(TextRankSummarizerParameters, parameters)
+        params: TextRankSummarizerParameters = cast(
+            TextRankSummarizerParameters, parameters
+        )
         nlp = get_nlp(params.lang, params.algo.value)
-        doc = self._check_sentences(nlp, params.algo.value, document)
+        doc = _check_sentences(nlp, params.algo.value, document)
         sent_bounds = [[s.start, s.end, set([])] for s in doc.sents]
         num_sentences = int_float(params.num_sentences)
-        limit_sentences = round(len(sent_bounds) * num_sentences) if isinstance(num_sentences, float) else num_sentences
+        limit_sentences = (
+            round(len(sent_bounds) * num_sentences)
+            if isinstance(num_sentences, float)
+            else num_sentences
+        )
         phrase_id = 0
         unit_vector = []
         for p in doc._.phrases:
@@ -130,27 +130,34 @@ class TextRankSummarizerFormatter(FormatterBase, ProcessorBase, ABC):
             if num_sent == limit_sentences:
                 break
         if params.preserve_order:
-            summary_sents = [v for k, v in sorted(summary_sentences.items(), key=itemgetter(0))]
+            summary_sents = [
+                v for k, v in sorted(summary_sentences.items(), key=itemgetter(0))
+            ]
         else:
             summary_sents = list(summary_sentences.values())
         return summary_sents
 
-    def format(self, document: Document, parameters: FormatterParameters) \
-            -> Response:
+    def format(self, document: Document, parameters: FormatterParameters) -> Response:
         summary_sentences = self._summarize(document, parameters)
-        summary = '\n'.join(summary_sentences)
+        summary = "\n".join(summary_sentences)
         return PlainTextResponse(summary)
 
-    def process(self, documents: List[Document], parameters: ProcessorParameters) \
-            -> List[Document]:
-        params: TextRankSummarizerParameters = \
-            cast(TextRankSummarizerParameters, parameters)
+    def process(
+        self, documents: List[Document], parameters: ProcessorParameters
+    ) -> List[Document]:
+        params: TextRankSummarizerParameters = cast(
+            TextRankSummarizerParameters, parameters
+        )
         for document in documents:
             summary_sentences = self._summarize(document, parameters)
-            if params.as_metadata is not None and len(params.as_metadata):
-                if not document.metadata:
-                    document.metadata = {}
-                document.metadata[params.as_metadata] = '\n'.join(summary_sentences)
+            summary = "\n".join(summary_sentences)
+            if params.as_altText is not None and len(params.as_altText):
+                document.altTexts = document.altTexts or []
+                altTexts = [
+                    alt for alt in document.altTexts if alt.name != params.as_altText
+                ]
+                altTexts.append(AltText(name=params.as_altText, text=summary))
+                document.altTexts = altTexts
             else:
                 document.text = ""
                 document.sentences = []
@@ -170,10 +177,17 @@ class TextRankSummarizerFormatter(FormatterBase, ProcessorBase, ABC):
 
 # Deprecated model shortcuts, only used in errors and warnings
 MODEL_SHORTCUTS = {
-    "en": "en_core_web_sm", "de": "de_core_news_sm", "es": "es_core_news_sm",
-    "pt": "pt_core_news_sm", "fr": "fr_core_news_sm", "it": "it_core_news_sm",
-    "nl": "nl_core_news_sm", "el": "el_core_news_sm", "nb": "nb_core_news_sm",
-    "lt": "lt_core_news_sm", "xx": "xx_ent_wiki_sm"
+    "en": "en_core_web_sm",
+    "de": "de_core_news_sm",
+    "es": "es_core_news_sm",
+    "pt": "pt_core_news_sm",
+    "fr": "fr_core_news_sm",
+    "it": "it_core_news_sm",
+    "nl": "nl_core_news_sm",
+    "el": "el_core_news_sm",
+    "nb": "nb_core_news_sm",
+    "lt": "lt_core_news_sm",
+    "xx": "xx_ent_wiki_sm",
 }
 
 
@@ -203,7 +217,8 @@ def load_spacy_model(model, *pip_args):
     compatibility = get_compatibility()
     if model_name not in compatibility:
         msg.fail(
-            f"No compatible package found for '{model}' (spaCy v{spacy.about.__version__}), fallback to blank model")
+            f"No compatible package found for '{model}' (spaCy v{spacy.about.__version__}), fallback to blank model"
+        )
         return spacy.blank(model_name)
     else:
         version = get_version(model_name, compatibility)
@@ -228,3 +243,28 @@ def require_package(name):
         return True
     except:  # noqa: E722
         return False
+
+
+def _check_sentences(nlp, algo: str, document: Document):
+    doc: Doc = None
+    if not document.sentences:
+        doc = nlp(document.text)
+    else:
+        textrank = nlp.get_pipe(algo)
+        parser = nlp.get_pipe("parser")
+        with nlp.select_pipes(disable=["parser", algo]):
+            doc = nlp(document.text)
+            sent_map = RangeMap()
+            for i, s in enumerate(document.sentences):
+                if s.end > s.start:
+                    sent_map[s.start : s.end] = i
+            current = -1
+            for token in doc:
+                if token.idx in sent_map:
+                    idsent = sent_map[token.idx]
+                    if idsent != current:
+                        token.is_sent_start = True
+                        current = idsent
+            if doc.has_annotation("SENT_START"):
+                doc = textrank(parser(doc))
+    return doc
